"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var portastic = require("portastic");
var appium_server_1 = require("./lib/appium-server");
var appium_driver_1 = require("./lib/appium-driver");
var ns_capabilities_1 = require("./lib/ns-capabilities");
var utils_1 = require("./lib/utils");
var appium_driver_2 = require("./lib/appium-driver");
exports.AppiumDriver = appium_driver_2.AppiumDriver;
var element_helper_1 = require("./lib/element-helper");
exports.ElementHelper = element_helper_1.ElementHelper;
var ui_element_1 = require("./lib/ui-element");
exports.UIElement = ui_element_1.UIElement;
var point_1 = require("./lib/point");
exports.Point = point_1.Point;
var locators_1 = require("./lib/locators");
exports.Locator = locators_1.Locator;
var emulator_manager_1 = require("./lib/emulator-manager");
exports.EmulatorManager = emulator_manager_1.EmulatorManager;
var nsCapabilities = new ns_capabilities_1.NsCapabilities();
var appiumServer = new appium_server_1.AppiumServer(nsCapabilities);
var appiumDriver = null;
function startServer(port) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var retry, _a, hasStarted, retryCount, tempPort;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    appiumServer.port = port || nsCapabilities.port;
                    retry = false;
                    if (!!appiumServer.port) return [3 /*break*/, 2];
                    _a = appiumServer;
                    return [4 /*yield*/, portastic.find({ min: 8600, max: 9080 })];
                case 1:
                    _a.port = (_b.sent())[0];
                    retry = true;
                    _b.label = 2;
                case 2: return [4 /*yield*/, appiumServer.start()];
                case 3:
                    hasStarted = _b.sent();
                    retryCount = 0;
                    _b.label = 4;
                case 4:
                    if (!(retry && !hasStarted && retryCount < 10)) return [3 /*break*/, 7];
                    tempPort = appiumServer.port + 10;
                    return [4 /*yield*/, portastic.find({ min: tempPort, max: 9180 })];
                case 5:
                    tempPort = (_b.sent())[0];
                    console.log("Trying to use port: ", tempPort);
                    appiumServer.port = tempPort;
                    return [4 /*yield*/, appiumServer.start()];
                case 6:
                    hasStarted = _b.sent();
                    retryCount++;
                    return [3 /*break*/, 4];
                case 7:
                    if (!hasStarted) {
                        throw new Error("Appium driver failed to start!!! Run with --verbose option for more info!");
                    }
                    appiumServer.hasStarted = hasStarted;
                    process.on("uncaughtException", function () { return utils_1.shutdown(appiumServer.server, nsCapabilities.verbose); });
                    process.on("exit", function () { return utils_1.shutdown(appiumServer.server, nsCapabilities.verbose); });
                    process.on("SIGINT", function () { return utils_1.shutdown(appiumServer.server, nsCapabilities.verbose); });
                    return [2 /*return*/];
            }
        });
    });
}
exports.startServer = startServer;
;
function stopServer() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!(appiumDriver !== null && appiumDriver.isAlive)) return [3 /*break*/, 2];
                    return [4 /*yield*/, appiumDriver.quit()];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2:
                    if (!(appiumServer !== null && appiumServer.hasStarted)) return [3 /*break*/, 4];
                    return [4 /*yield*/, appiumServer.stop()];
                case 3:
                    _a.sent();
                    _a.label = 4;
                case 4: return [2 /*return*/];
            }
        });
    });
}
exports.stopServer = stopServer;
;
function createDriver() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!appiumServer.hasStarted) {
                        throw new Error("Server is not available!");
                    }
                    if (!nsCapabilities.appiumCapsLocation) {
                        throw new Error("Provided path to appium capabilities is not correct!");
                    }
                    if (!nsCapabilities.runType) {
                        throw new Error("--runType is missing! Make sure it is provided correctly! It is used to parse the configuration for appium driver!");
                    }
                    if (!(appiumDriver !== null && appiumDriver.isAlive)) return [3 /*break*/, 1];
                    return [2 /*return*/, appiumDriver];
                case 1:
                    if (!(appiumDriver === null)) return [3 /*break*/, 3];
                    return [4 /*yield*/, appium_driver_1.AppiumDriver.createAppiumDriver(appiumServer.port, nsCapabilities)];
                case 2:
                    appiumDriver = _a.sent();
                    return [3 /*break*/, 5];
                case 3:
                    if (!(appiumDriver !== null && !appiumDriver.isAlive)) return [3 /*break*/, 5];
                    return [4 /*yield*/, appiumDriver.inint()];
                case 4:
                    _a.sent();
                    _a.label = 5;
                case 5: return [2 /*return*/, appiumDriver];
            }
        });
    });
}
exports.createDriver = createDriver;
