"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
require('colors');
var chai = require("chai");
var wd = require("wd");
var chaiAsPromised = require("chai-as-promised");
chai.use(chaiAsPromised);
exports.should = chai.should();
chaiAsPromised.transferPromiseness = wd.transferPromiseness;
var element_helper_1 = require("./element-helper");
var ui_element_1 = require("./ui-element");
var locators_1 = require("./locators");
var utils_1 = require("./utils");
var image_helper_1 = require("./image-helper");
var emulator_manager_1 = require("./emulator-manager");
var fs_1 = require("fs");
var webdriverio = require("webdriverio");
var AppiumDriver = /** @class */ (function () {
    function AppiumDriver(_driver, _wd, webio, _driverConfig, _args) {
        this._driver = _driver;
        this._wd = _wd;
        this.webio = webio;
        this._driverConfig = _driverConfig;
        this._args = _args;
        this._isAlive = false;
        this._elementHelper = new element_helper_1.ElementHelper(this._args.appiumCaps.platformName.toLowerCase(), this._args.appiumCaps.platformVersion.toLowerCase());
        this._imageHelper = new image_helper_1.ImageHelper();
        this._isAlive = true;
        this._locators = new locators_1.Locator(this._args.appiumCaps.platformName, this._args.appiumCaps.platformVersion);
        this.webio.requestHandler.sessionID = this._driver.sessionID;
    }
    Object.defineProperty(AppiumDriver.prototype, "capabilities", {
        get: function () {
            return this._args.appiumCaps;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppiumDriver.prototype, "platformName", {
        get: function () {
            return this._args.appiumCaps.platformName;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppiumDriver.prototype, "platformVesrion", {
        get: function () {
            return this._args.appiumCaps.platformVesrion;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppiumDriver.prototype, "elementHelper", {
        get: function () {
            return this._elementHelper;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppiumDriver.prototype, "locators", {
        get: function () {
            return this._locators;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppiumDriver.prototype, "isAlive", {
        get: function () {
            return this._isAlive;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppiumDriver.prototype, "driver", {
        get: function () {
            return this._driver;
        },
        enumerable: true,
        configurable: true
    });
    AppiumDriver.prototype.wdio = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.webio];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    AppiumDriver.prototype.wd = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._wd];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    AppiumDriver.prototype.click = function (args) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.webio.click(args)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    AppiumDriver.prototype.navBack = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._driver.back()];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     *
     * @param xPath
     * @param waitForElement
     */
    AppiumDriver.prototype.findElementByXPath = function (xPath, waitForElement) {
        if (waitForElement === void 0) { waitForElement = AppiumDriver.defaultWaitTime; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var searchM, _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        searchM = "waitForElementByXPath";
                        _a = ui_element_1.UIElement.bind;
                        return [4 /*yield*/, this._driver.waitForElementByXPath(xPath, waitForElement)];
                    case 1: return [4 /*yield*/, new (_a.apply(ui_element_1.UIElement, [void 0, _b.sent(), this._driver, this._wd, this.webio, searchM, xPath]))()];
                    case 2: return [2 /*return*/, _b.sent()];
                }
            });
        });
    };
    /**
     *
     * @param xPath
     * @param waitForElement
     */
    AppiumDriver.prototype.findElementsByXPath = function (xPath, waitForElement) {
        if (waitForElement === void 0) { waitForElement = AppiumDriver.defaultWaitTime; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this.convertArrayToUIElements;
                        return [4 /*yield*/, this._driver.waitForElementsByXPath(xPath, waitForElement)];
                    case 1: return [4 /*yield*/, _a.apply(this, [_b.sent(), "waitForElementByXPath", xPath])];
                    case 2: return [2 /*return*/, _b.sent()];
                }
            });
        });
    };
    /**
     * Search for element by given text. The seacrch is case insensitive for android
     * @param text
     * @param match
     * @param waitForElement
     */
    AppiumDriver.prototype.findElementByText = function (text, match, waitForElement) {
        if (match === void 0) { match = 0 /* exact */; }
        if (waitForElement === void 0) { waitForElement = AppiumDriver.defaultWaitTime; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var shouldMatch;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        shouldMatch = match === 0 /* exact */ ? true : false;
                        return [4 /*yield*/, this.findElementByXPath(this._elementHelper.getXPathByText(text, shouldMatch), waitForElement)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Search for elements by given text. The seacrch is case insensitive for android
     * @param text
     * @param match
     * @param waitForElement
     */
    AppiumDriver.prototype.findElementsByText = function (text, match, waitForElement) {
        if (match === void 0) { match = 0 /* exact */; }
        if (waitForElement === void 0) { waitForElement = AppiumDriver.defaultWaitTime; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var shouldMatch;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        shouldMatch = match === 0 /* exact */ ? true : false;
                        return [4 /*yield*/, this.findElementsByXPath(this._elementHelper.getXPathByText(text, shouldMatch), waitForElement)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Searches for element by element native class name like button, textView etc which will be translated to android.widgets.Button or XCUIElementTypeButton (iOS 10 and higher) or UIElementButton (iOS 9)
     * Notice this is not the same as css class
     * @param className
     * @param waitForElement
     */
    AppiumDriver.prototype.findElementByClassName = function (className, waitForElement) {
        if (waitForElement === void 0) { waitForElement = AppiumDriver.defaultWaitTime; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = ui_element_1.UIElement.bind;
                        return [4 /*yield*/, this._driver.waitForElementByClassName(className, waitForElement)];
                    case 1: return [2 /*return*/, new (_a.apply(ui_element_1.UIElement, [void 0, _b.sent(), this._driver, this._wd, this.webio, "waitForElementByClassName", className]))()];
                }
            });
        });
    };
    /**
     * Searches for element by element native class name like button, textView etc which will be translated to android.widgets.Button or XCUIElementTypeButton (iOS 10 and higher) or UIElementButton (iOS 9)
     * Notice this is not the same as css class
     * @param className
     * @param waitForElement
     */
    AppiumDriver.prototype.findElementsByClassName = function (className, waitForElement) {
        if (waitForElement === void 0) { waitForElement = AppiumDriver.defaultWaitTime; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this.convertArrayToUIElements;
                        return [4 /*yield*/, this._driver.waitForElementsByClassName(className, waitForElement)];
                    case 1: return [4 /*yield*/, _a.apply(this, [_b.sent(), "waitForElementByClassName", className])];
                    case 2: return [2 /*return*/, _b.sent()];
                }
            });
        });
    };
    /**
     * Find element by automationText
     * @param id
     * @param waitForElement
     */
    AppiumDriver.prototype.findElementByAccessibilityId = function (id, waitForElement) {
        if (waitForElement === void 0) { waitForElement = AppiumDriver.defaultWaitTime; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = ui_element_1.UIElement.bind;
                        return [4 /*yield*/, this._driver.waitForElementByAccessibilityId(id, waitForElement)];
                    case 1: return [2 /*return*/, new (_a.apply(ui_element_1.UIElement, [void 0, _b.sent(), this._driver, this._wd, this.webio, "waitForElementByAccessibilityId", id]))()];
                }
            });
        });
    };
    /**
     * Find elements by automationText
     * @param id
     * @param waitForElement
     */
    AppiumDriver.prototype.findElementsByAccessibilityId = function (id, waitForElement) {
        if (waitForElement === void 0) { waitForElement = AppiumDriver.defaultWaitTime; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this.convertArrayToUIElements;
                        return [4 /*yield*/, this._driver.waitForElementsByAccessibilityId(id, waitForElement)];
                    case 1: return [4 /*yield*/, _a.apply(this, [_b.sent(), "waitForElementsByAccessibilityId", id])];
                    case 2: return [2 /*return*/, _b.sent()];
                }
            });
        });
    };
    /**
     * Scrolls from point to other point with minimum inertia
     * @param y
     * @param x
     * @param yOffset
     * @param duration
     * @param xOffset
     */
    AppiumDriver.prototype.scroll = function (direction, SwipeDirection, y, x, yOffset, xOffset) {
        if (xOffset === void 0) { xOffset = 0; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                utils_1.scroll(this._wd, this._driver, direction, this.webio.isIOS, y, x, yOffset, xOffset, this._args.verbose);
                return [2 /*return*/];
            });
        });
    };
    /**
     *
     * @param direction
     * @param SwipeDirection
     * @param element
     * @param startPoint
     * @param yOffset
     * @param xOffset
     * @param retryCount
     */
    AppiumDriver.prototype.scrollTo = function (direction, element, startPoint, yOffset, xOffset, retryCount) {
        if (xOffset === void 0) { xOffset = 0; }
        if (retryCount === void 0) { retryCount = 7; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var el, error_1, _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        el = null;
                        _b.label = 1;
                    case 1:
                        if (!(el === null && retryCount > 0)) return [3 /*break*/, 12];
                        _b.label = 2;
                    case 2:
                        _b.trys.push([2, 7, , 9]);
                        return [4 /*yield*/, element()];
                    case 3:
                        el = _b.sent();
                        return [4 /*yield*/, el.isDisplayed()];
                    case 4:
                        if (!!(_b.sent())) return [3 /*break*/, 6];
                        return [4 /*yield*/, utils_1.scroll(this._wd, this._driver, direction, this.webio.isIOS, startPoint.y, startPoint.x, yOffset, xOffset, this._args.verbose)];
                    case 5:
                        _b.sent();
                        _b.label = 6;
                    case 6: return [3 /*break*/, 9];
                    case 7:
                        error_1 = _b.sent();
                        return [4 /*yield*/, utils_1.scroll(this._wd, this._driver, direction, this.webio.isIOS, startPoint.y, startPoint.x, yOffset, xOffset, this._args.verbose)];
                    case 8:
                        _b.sent();
                        return [3 /*break*/, 9];
                    case 9:
                        _a = el !== null;
                        if (!_a) return [3 /*break*/, 11];
                        return [4 /*yield*/, el.isDisplayed()];
                    case 10:
                        _a = (_b.sent());
                        _b.label = 11;
                    case 11:
                        if (_a) {
                            return [3 /*break*/, 12];
                        }
                        else {
                            el = null;
                        }
                        retryCount--;
                        return [3 /*break*/, 1];
                    case 12: return [2 /*return*/, el];
                }
            });
        });
    };
    /**
     * Swipe from point with offset and inertia according to duatio
     * @param y
     * @param x
     * @param yOffset
     * @param inertia
     * @param xOffset
     */
    AppiumDriver.prototype.swipe = function (y, x, yOffset, inertia, xOffset) {
        if (inertia === void 0) { inertia = 250; }
        if (xOffset === void 0) { xOffset = 0; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var direction, action;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        direction = 1;
                        if (this.webio.isIOS) {
                            direction = -1;
                        }
                        action = new this._wd.TouchAction(this._driver);
                        action
                            .press({ x: x, y: y })
                            .wait(inertia)
                            .moveTo({ x: xOffset, y: direction * yOffset })
                            .release();
                        return [4 /*yield*/, action.perform()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this._driver.sleep(150)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    AppiumDriver.prototype.source = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.webio.source()];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    AppiumDriver.prototype.sessionId = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.driver.getSessionId()];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    AppiumDriver.prototype.compareScreen = function (imageName, timeOutSeconds, tollerance) {
        if (timeOutSeconds === void 0) { timeOutSeconds = 3; }
        if (tollerance === void 0) { tollerance = 0.01; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var expectedImage, eventStartTime, counter, actualImage_1, actualImage, diffImage, result, eventStartTime, counter, actualImage_2;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!imageName.endsWith(AppiumDriver.pngFileExt)) {
                            imageName = imageName.concat(AppiumDriver.pngFileExt);
                        }
                        if (!this._storage) {
                            this._storage = utils_1.getStorage(this._args);
                        }
                        if (!this._logPath) {
                            this._logPath = utils_1.getReportPath(this._args);
                        }
                        expectedImage = utils_1.resolve(this._storage, imageName);
                        if (!!utils_1.fileExists(expectedImage)) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.takeScreenshot(utils_1.resolve(this._storage, imageName.replace(".", "_actual.")))];
                    case 1:
                        _a.sent();
                        console.log("Remove the 'actual' suffix to continue using the image as expected one ", expectedImage);
                        eventStartTime = Date.now().valueOf();
                        counter = 1;
                        timeOutSeconds *= 1000;
                        _a.label = 2;
                    case 2:
                        if (!((Date.now().valueOf() - eventStartTime) <= timeOutSeconds)) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.takeScreenshot(utils_1.resolve(this._logPath, imageName.replace(".", "_actual" + "_" + counter + ".")))];
                    case 3:
                        actualImage_1 = _a.sent();
                        counter++;
                        return [3 /*break*/, 2];
                    case 4: return [2 /*return*/, false];
                    case 5: return [4 /*yield*/, this.takeScreenshot(utils_1.resolve(this._logPath, imageName.replace(".", "_actual.")))];
                    case 6:
                        actualImage = _a.sent();
                        diffImage = actualImage.replace("actual", "diff");
                        return [4 /*yield*/, this._imageHelper.compareImages(actualImage, expectedImage, diffImage, tollerance)];
                    case 7:
                        result = _a.sent();
                        if (!!result) return [3 /*break*/, 12];
                        eventStartTime = Date.now().valueOf();
                        counter = 1;
                        timeOutSeconds *= 1000;
                        _a.label = 8;
                    case 8:
                        if (!((Date.now().valueOf() - eventStartTime) <= timeOutSeconds && !result)) return [3 /*break*/, 11];
                        return [4 /*yield*/, this.takeScreenshot(utils_1.resolve(this._logPath, imageName.replace(".", "_actual" + "_" + counter + ".")))];
                    case 9:
                        actualImage_2 = _a.sent();
                        return [4 /*yield*/, this._imageHelper.compareImages(actualImage_2, expectedImage, diffImage, tollerance)];
                    case 10:
                        result = _a.sent();
                        counter++;
                        return [3 /*break*/, 8];
                    case 11: return [3 /*break*/, 13];
                    case 12:
                        if (utils_1.fileExists(diffImage)) {
                            fs_1.unlinkSync(diffImage);
                        }
                        if (utils_1.fileExists(actualImage)) {
                            fs_1.unlinkSync(actualImage);
                        }
                        _a.label = 13;
                    case 13: return [2 /*return*/, result];
                }
            });
        });
    };
    AppiumDriver.prototype.takeScreenshot = function (fileName) {
        var _this = this;
        if (!fileName.endsWith(AppiumDriver.pngFileExt)) {
            fileName = fileName.concat(AppiumDriver.pngFileExt);
        }
        return new Promise(function (resolve, reject) {
            _this._driver.takeScreenshot().then(function (image, err) {
                fs_1.writeFileSync(fileName, image, 'base64');
                resolve(fileName);
            });
        });
    };
    AppiumDriver.prototype.logScreenshoot = function (fileName) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var imgPath;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this._logPath && !utils_1.fileExists(fileName)) {
                            this._logPath = utils_1.getReportPath(this._args);
                        }
                        if (!fileName.endsWith(AppiumDriver.pngFileExt)) {
                            fileName = fileName.concat(AppiumDriver.pngFileExt);
                        }
                        return [4 /*yield*/, this.takeScreenshot(utils_1.resolve(this._logPath, fileName))];
                    case 1:
                        imgPath = _a.sent();
                        return [2 /*return*/, imgPath];
                }
            });
        });
    };
    AppiumDriver.createAppiumDriver = function (port, args) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var driverConfig, driver, sauceUser, sauceKey, webio;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        driverConfig = {
                            host: "localhost",
                            port: port
                        };
                        if (args.appiumCaps.platformName.toLowerCase() === "android") {
                        }
                        return [4 /*yield*/, emulator_manager_1.EmulatorManager.startEmulator(args)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, wd.promiseChainRemote(driverConfig)];
                    case 2:
                        driver = _a.sent();
                        AppiumDriver.configureLogging(driver, args.verbose);
                        if (!args.appiumCaps.app) {
                            utils_1.log("Getting caps.app!", args.verbose);
                            args.appiumCaps.app = utils_1.getAppPath(args.appiumCaps.platformName.toLowerCase(), args.runType.toLowerCase());
                        }
                        if (args.isSauceLab) {
                            sauceUser = process.env.SAUCE_USER;
                            sauceKey = process.env.SAUCE_KEY;
                            if (!sauceKey || !sauceUser) {
                                throw new Error("Sauce Labs Username or Access Key is missing! Check environment variables for SAUCE_USER and SAUCE_KEY !!!");
                            }
                            driverConfig = {
                                host: "https://" + sauceUser + ":" + sauceKey + "@ondemand.saucelabs.com:443/wd/hub"
                            };
                            args.appiumCaps.app = "sauce-storage:" + args.appiumCaps.app;
                        }
                        utils_1.log("Creating driver!", args.verbose);
                        webio = webdriverio.remote({
                            baseUrl: driverConfig.host,
                            port: driverConfig.port,
                            logLevel: 'warn',
                            desiredCapabilities: args.appiumCaps
                        });
                        return [4 /*yield*/, driver.init(args.appiumCaps)];
                    case 3:
                        _a.sent();
                        return [2 /*return*/, new AppiumDriver(driver, wd, webio, driverConfig, args)];
                }
            });
        });
    };
    AppiumDriver.prototype.inint = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._driver.init(this._args.appiumCaps)];
                    case 1:
                        _a.sent();
                        this.webio.requestHandler.sessionID = this._driver.sessionID;
                        this._isAlive = true;
                        return [2 /*return*/];
                }
            });
        });
    };
    AppiumDriver.prototype.quit = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var error_2;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        console.log("Killing driver");
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 4, , 5]);
                        return [4 /*yield*/, emulator_manager_1.EmulatorManager.stop(this._args)];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, this._driver.quit()];
                    case 3:
                        _a.sent();
                        return [3 /*break*/, 5];
                    case 4:
                        error_2 = _a.sent();
                        return [3 /*break*/, 5];
                    case 5:
                        this._isAlive = false;
                        console.log("Driver is dead");
                        return [2 /*return*/];
                }
            });
        });
    };
    AppiumDriver.prototype.convertArrayToUIElements = function (array, searchM, args) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var i, arrayOfUIElements;
            return tslib_1.__generator(this, function (_a) {
                i = 0;
                arrayOfUIElements = new Array();
                if (!array || array === null) {
                    return [2 /*return*/, arrayOfUIElements];
                }
                array.forEach(function (element) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                    var _a, _b, _c;
                    return tslib_1.__generator(this, function (_d) {
                        switch (_d.label) {
                            case 0:
                                _b = (_a = arrayOfUIElements).push;
                                _c = ui_element_1.UIElement.bind;
                                return [4 /*yield*/, element];
                            case 1:
                                _b.apply(_a, [new (_c.apply(ui_element_1.UIElement, [void 0, _d.sent(), this._driver, this._wd, this.wdio, searchM, args, i]))()]);
                                i++;
                                return [2 /*return*/];
                        }
                    });
                }); });
                return [2 /*return*/, arrayOfUIElements];
            });
        });
    };
    AppiumDriver.configureLogging = function (driver, verbose) {
        driver.on("status", function (info) {
            utils_1.log(info.cyan, verbose);
        });
        driver.on("command", function (meth, path, data) {
            utils_1.log(" > " + meth.yellow + path.grey + " " + (data || ""), verbose);
        });
        driver.on("http", function (meth, path, data) {
            utils_1.log(" > " + meth.magenta + path + " " + (data || "").grey, verbose);
        });
    };
    ;
    AppiumDriver.defaultWaitTime = 5000;
    AppiumDriver.pngFileExt = '.png';
    AppiumDriver.partialUrl = "/wd/hub/session/";
    AppiumDriver._devices = new Map();
    return AppiumDriver;
}());
exports.AppiumDriver = AppiumDriver;
