"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var child_process = require("child_process");
var utils_1 = require("./utils");
var AppiumServer = /** @class */ (function () {
    function AppiumServer(_args) {
        this._args = _args;
        this._runType = this._args.runType;
        this._hasStarted = false;
        this.resolveAppiumDependency();
    }
    Object.defineProperty(AppiumServer.prototype, "port", {
        get: function () {
            return this._port;
        },
        set: function (port) {
            this._port = port;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppiumServer.prototype, "runType", {
        get: function () {
            return this._runType;
        },
        set: function (runType) {
            this._runType = runType;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppiumServer.prototype, "server", {
        get: function () {
            return this._server;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppiumServer.prototype, "hasStarted", {
        get: function () {
            return this._hasStarted;
        },
        set: function (hasStarted) {
            this._hasStarted = hasStarted;
        },
        enumerable: true,
        configurable: true
    });
    AppiumServer.prototype.start = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var logLevel, responce;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        utils_1.log("Starting server...", this._args.verbose);
                        logLevel = this._args.verbose === true ? "debug" : "info";
                        this._server = child_process.spawn(this._appium, ["-p", this.port.toString(), "--log-level", logLevel], {
                            shell: true,
                            detached: false
                        });
                        return [4 /*yield*/, utils_1.waitForOutput(this._server, /listener started/, /Error: listen/, 60000, this._args.verbose)];
                    case 1:
                        responce = _a.sent();
                        return [2 /*return*/, responce];
                }
            });
        });
    };
    AppiumServer.prototype.stop = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this._server.on("close", function (code, signal) {
                utils_1.log("Appium terminated due signal: " + signal + " and code: " + code, _this._args.verbose);
                resolve();
            });
            _this._server.on("exit", function (code, signal) {
                utils_1.log("Appium terminated due signal: " + signal + " and code: " + code, _this._args.verbose);
                console.log(_this._server.killed);
                resolve();
            });
            _this._server.on("error", function (code, signal) {
                utils_1.log("Appium terminated due signal: " + signal + " and code: " + code, _this._args.verbose);
                resolve();
            });
            _this._server.on("disconnect", function (code, signal) {
                utils_1.log("Appium terminated due signal: " + signal + " and code: " + code, _this._args.verbose);
                resolve();
            });
            utils_1.log("Stopping server...", _this._args.verbose);
            try {
                if (utils_1.isWin) {
                    utils_1.shutdown(_this._server, _this._args.verbose);
                }
                else {
                    _this._server.kill("SIGINT");
                    utils_1.shutdown(_this._server, _this._args.verbose);
                    //this._server.kill("SIGKILL");
                }
            }
            catch (error) {
                console.log(error);
            }
        });
    };
    // Resolve appium dependency
    AppiumServer.prototype.resolveAppiumDependency = function () {
        var projectDir = this._args.projectDir;
        var pluginBinary = this._args.pluginBinary;
        var projectBinary = this._args.projectBinary;
        var pluginRoot = this._args.pluginRoot;
        var appium = process.platform === "win32" ? "appium.cmd" : "appium";
        var pluginAppiumBinary = utils_1.resolve(pluginBinary, appium);
        var projectAppiumBinary = utils_1.resolve(projectBinary, appium);
        if (utils_1.fileExists(pluginAppiumBinary)) {
            utils_1.log("Using plugin-local Appium binary.", this._args.verbose);
            appium = pluginAppiumBinary;
        }
        else if (utils_1.fileExists(projectAppiumBinary)) {
            utils_1.log("Using project-local Appium binary.", this._args.verbose);
            appium = projectAppiumBinary;
        }
        else {
            utils_1.log("Using global Appium binary.", this._args.verbose);
        }
        this._appium = appium;
    };
    return AppiumServer;
}());
exports.AppiumServer = AppiumServer;
