"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var fs_1 = require("fs");
var utils = require("./utils");
var parser = require("./parser");
function resolveCapabilities(capsLocation, runType, projectDir, verbose) {
    if (verbose === void 0) { verbose = false; }
    var caps;
    var customCapabilitiesConfigs = searchCustomCapabilities(capsLocation, projectDir);
    if (customCapabilitiesConfigs) {
        var customCapabilities = JSON.parse(customCapabilitiesConfigs);
        utils.log(customCapabilities, verbose);
        caps = customCapabilities[runType];
        if (!caps) {
            throw new Error("Not suitable runType!!!");
        }
    }
    else {
        throw new Error("No capabilities found!!!");
    }
    return caps;
}
exports.resolveCapabilities = resolveCapabilities;
function searchCustomCapabilities(capabilitiesLocation, projectDir, verbose) {
    if (verbose === void 0) { verbose = false; }
    // resolve capabilites if exist
    if (utils.fileExists(capabilitiesLocation) && utils.isFile(capabilitiesLocation)) {
        return setCustomCapabilities(capabilitiesLocation, verbose);
    }
    // search for default capabilites
    var customCapabilitiesLocation = utils.searchFiles(projectDir, parser.capabilitiesName)[0];
    if (utils.fileExists(customCapabilitiesLocation)) {
        return setCustomCapabilities(customCapabilitiesLocation, verbose);
    }
    // search in parent folder for capabilities
    var appParentFolder = path_1.dirname(projectDir);
    customCapabilitiesLocation = utils.searchFiles(appParentFolder, parser.capabilitiesName, true)[0];
    if (utils.fileExists(customCapabilitiesLocation)) {
        return setCustomCapabilities(customCapabilitiesLocation, verbose);
    }
    else {
        // search for capabilities recursive 
        var cap = utils.searchFiles(appParentFolder, parser.capabilitiesName, verbose)[0];
        if (cap) {
            setCustomCapabilities(cap, verbose);
        }
    }
    throw Error("No capabilities found!!!");
}
exports.searchCustomCapabilities = searchCustomCapabilities;
function setCustomCapabilities(appiumCapabilitiesLocation, verbose) {
    if (verbose === void 0) { verbose = false; }
    var file = fs_1.readFileSync(appiumCapabilitiesLocation);
    process.env.APPIUM_CAPABILITIES = file;
    utils.log("Custom capabilities found at: " + appiumCapabilitiesLocation, verbose);
    return file;
}
