"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var utils_1 = require("./utils");
var child_process = require("child_process");
var ANDROID_HOME = process.env["ANDROID_HOME"];
var EMULATOR = utils_1.resolve(ANDROID_HOME, "emulator", "emulator");
var ADB = utils_1.resolve(ANDROID_HOME, "platform-tools", "adb");
var EmulatorManager = /** @class */ (function () {
    function EmulatorManager(_id, _args, _emulatorProc, _shouldKill) {
        this._id = _id;
        this._args = _args;
        this._emulatorProc = _emulatorProc;
        this._shouldKill = _shouldKill;
    }
    Object.defineProperty(EmulatorManager.prototype, "id", {
        get: function () {
            return this._id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EmulatorManager.prototype, "emulatorProc", {
        get: function () {
            return this._emulatorProc;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EmulatorManager.prototype, "args", {
        get: function () {
            return this._args;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EmulatorManager.prototype, "shouldKill", {
        get: function () {
            return this._shouldKill;
        },
        enumerable: true,
        configurable: true
    });
    EmulatorManager.startEmulator = function (args) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var id, emulator, responce;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!args.appiumCaps.avd || args.appiumCaps.avd === "") {
                            utils_1.log("No avd name provided! We will not start the emulator!", args.verbose);
                            return [2 /*return*/, false];
                        }
                        if (EmulatorManager._emulatorIds.size === 0) {
                            EmulatorManager.loadEmulatorsIds();
                        }
                        id = EmulatorManager.emulatorId(args.appiumCaps.platformVersion) || "5554";
                        emulator = child_process.spawn(EMULATOR, ["-avd ", args.appiumCaps.avd, "-port ", id, args.emulatorOptions], {
                            shell: true,
                            detached: false
                        });
                        return [4 /*yield*/, utils_1.waitForOutput(emulator, new RegExp(args.appiumCaps.avd, "i"), new RegExp("Error", "i"), args.appiumCaps.lt || 180000, args.verbose)];
                    case 1:
                        responce = _a.sent();
                        if (responce === true) {
                            EmulatorManager.waitUntilEmulatorBoot(id, args.appiumCaps.lt || 180000);
                            EmulatorManager._emulators.set(args.runType, new EmulatorManager(id, args, emulator, true));
                        }
                        else {
                            utils_1.log("Emulator is probably already started!", args.verbose);
                        }
                        return [2 /*return*/, responce];
                }
            });
        });
    };
    EmulatorManager.stop = function (args) {
        if (EmulatorManager._emulators.has(args.runType)) {
            var emu = EmulatorManager._emulators.get(args.runType);
            utils_1.executeCommand(ADB + " -s emulator-" + emu.id + " emu kill");
            // return new Promise((resolve, reject) => {
            //     emu.emulatorProc.on("close", (code, signal) => {
            //         log(`Emulator terminated due signal: ${signal} and code: ${code}`, args.verbose);
            //         resolve();
            //     });
            //     emu.emulatorProc.on("exit", (code, signal) => {
            //         log(`Emulator terminated due signal: ${signal} and code: ${code}`, args.verbose);
            //         resolve();
            //     });
            //     emu.emulatorProc.on("error", (code, signal) => {
            //         log(`Emulator terminated due signal: ${signal} and code: ${code}`, args.verbose);
            //         resolve();
            //     });
            //     emu.emulatorProc.on("disconnect", (code, signal) => {
            //         log(`Emulator terminated due signal: ${signal} and code: ${code}`, args.verbose);
            //         resolve();
            //     });
            //     try {
            //         if (isWin) {
            //             shutdown(emu.emulatorProc, args.verbose);
            //         } else {
            //             executeCommand(ADB + " -s emulator-" + emu.id + " emu kill");
            //             shutdown(emu.emulatorProc, args.verbose);
            //             emu.emulatorProc.kill("SIGINT");
            //             //this._emulator.kill("SIGKILL");
            //         }
            //     } catch (error) {
            //         console.log(error);
            //     }
            // });
        }
    };
    EmulatorManager.waitUntilEmulatorBoot = function (deviceId, timeOut) {
        var startTime = new Date().getTime();
        var currentTime = new Date().getTime();
        var found = false;
        console.log("Booting emulator ...");
        while ((currentTime - startTime) < timeOut * 1000 && !found) {
            currentTime = new Date().getTime();
            found = this.checkIfEmulatorIsRunning("emulator-" + deviceId);
        }
        if (!found) {
            var error = deviceId + " failed to boot in " + timeOut + " seconds.";
            console.log(error, true);
        }
        else {
            console.log("Emilator is booted!");
        }
    };
    /**
     *
     * @param deviceId
     */
    EmulatorManager.checkIfEmulatorIsRunning = function (deviceId) {
        var isBooted = utils_1.executeCommand(ADB + " -s " + deviceId + " shell getprop sys.boot_completed").trim() === "1";
        if (isBooted) {
            isBooted = utils_1.executeCommand(ADB + " -s " + deviceId + " shell getprop init.svc.bootanim").toLowerCase().trim() === "stopped";
        }
        return isBooted;
        // let hasBooted = false;
        // let rowData = executeCommand(ADB + " -s " + deviceId + " shell dumpsys activity");
        // let list = rowData.split("\\r\\n");
        // list.forEach(line => {
        //     if (line.includes("Recent #0")
        //         && (line.includes("com.android.launcher")
        //             || line.includes("com.google.android.googlequicksearchbox")
        //             || line.includes("com.google.android.apps.nexuslauncher")
        //             || line.includes(deviceId))) {
        //         hasBooted = true;
        //     }
        // });
        // return hasBooted;
    };
    EmulatorManager.emulatorId = function (platformVersion) {
        return EmulatorManager._emulatorIds.get(platformVersion);
    };
    EmulatorManager.loadEmulatorsIds = function () {
        EmulatorManager._emulatorIds.set("4.2", "5554");
        EmulatorManager._emulatorIds.set("4.3", "5556");
        EmulatorManager._emulatorIds.set("4.4", "5558");
        EmulatorManager._emulatorIds.set("5.0", "5560");
        EmulatorManager._emulatorIds.set("6.0", "5562");
        EmulatorManager._emulatorIds.set("7.0", "5564");
        EmulatorManager._emulatorIds.set("7.1", "5566");
        EmulatorManager._emulatorIds.set("8.0", "5568");
    };
    EmulatorManager._emulators = new Map();
    EmulatorManager._emulatorIds = new Map();
    return EmulatorManager;
}());
exports.EmulatorManager = EmulatorManager;
