"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var blinkDiff = require("blink-diff");
var image_options_1 = require("./image-options");
var ImageHelper = /** @class */ (function () {
    function ImageHelper() {
    }
    ImageHelper.prototype.imageOutputLimit = function () {
        return image_options_1.ImageOptions.outputAll;
    };
    ImageHelper.prototype.thresholdType = function () {
        return image_options_1.ImageOptions.pixel;
    };
    ImageHelper.prototype.threshold = function () {
        return 0.1; // 0.1 percent; 500 percent
    };
    ImageHelper.prototype.delta = function () {
        return 20;
    };
    ImageHelper.prototype.cropImageA = function () {
        return { x: 0, y: 40 }; // Android = 35, iOS = 40
    };
    ImageHelper.prototype.cropImageB = function () {
        return { x: 0, y: 40 }; // Android = 35, iOS = 40
    };
    ImageHelper.prototype.verbose = function () {
        return false;
    };
    ImageHelper.prototype.runDiff = function (diffOptions, diffImage) {
        return new Promise(function (resolve, reject) {
            diffOptions.run(function (error, result) {
                if (error) {
                    throw error;
                }
                else {
                    var message = void 0;
                    var resultCode = diffOptions.hasPassed(result.code);
                    if (resultCode) {
                        message = "Screen compare passed!";
                        console.log(message);
                        console.log('Found ' + result.differences + ' differences.');
                        return resolve(true);
                    }
                    else {
                        message = "Screen compare failed!";
                        console.log(message);
                        console.log('Found ' + result.differences + ' differences.');
                        console.log('Diff image: ' + diffImage);
                        return resolve(false);
                    }
                }
            });
        });
    };
    ImageHelper.prototype.compareImages = function (actual, expected, output, valueThreshold, typeThreshold) {
        if (valueThreshold === void 0) { valueThreshold = this.threshold(); }
        if (typeThreshold === void 0) { typeThreshold = image_options_1.ImageOptions.pixel; }
        var diff = new blinkDiff({
            imageAPath: actual,
            imageBPath: expected,
            imageOutputPath: output,
            imageOutputLimit: this.imageOutputLimit(),
            thresholdType: typeThreshold,
            threshold: valueThreshold,
            delta: this.delta(),
            cropImageA: this.cropImageA(),
            cropImageB: this.cropImageB(),
            verbose: this.verbose(),
        });
        return this.runDiff(diff, output);
    };
    return ImageHelper;
}());
exports.ImageHelper = ImageHelper;
