"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var point_1 = require("./point");
var utils_1 = require("./utils");
var UIElement = /** @class */ (function () {
    function UIElement(_element, _driver, _wd, _webio, _searchMethod, _searchParams, _index) {
        this._element = _element;
        this._driver = _driver;
        this._wd = _wd;
        this._webio = _webio;
        this._searchMethod = _searchMethod;
        this._searchParams = _searchParams;
        this._index = _index;
    }
    /**
     * Click on element
     */
    UIElement.prototype.click = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.element()];
                    case 1: return [4 /*yield*/, (_a.sent()).click()];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Tap on element
     */
    UIElement.prototype.tap = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.element()];
                    case 1: return [4 /*yield*/, (_a.sent()).tap()];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * double tap
     */
    UIElement.prototype.doubleTap = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a, _b, _c, _d;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        _b = (_a = this._driver).execute;
                        _c = ['mobile: doubleTap'];
                        _d = {};
                        return [4 /*yield*/, this.element()];
                    case 1: return [4 /*yield*/, _b.apply(_a, _c.concat([(_d.element = (_e.sent()).value.ELEMENT, _d)]))];
                    case 2: return [2 /*return*/, _e.sent()];
                }
            });
        });
    };
    /**
     * Get location of element
     */
    UIElement.prototype.location = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var location, point;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.element()];
                    case 1: return [4 /*yield*/, (_a.sent()).getLocation()];
                    case 2:
                        location = _a.sent();
                        point = new point_1.Point(location.x, location.y);
                        return [2 /*return*/, point];
                }
            });
        });
    };
    /**
     * Get size of element
     */
    UIElement.prototype.size = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var size, point;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.element()];
                    case 1: return [4 /*yield*/, (_a.sent()).getSize()];
                    case 2:
                        size = _a.sent();
                        point = new point_1.Point(size.height, size.width);
                        return [2 /*return*/, point];
                }
            });
        });
    };
    UIElement.prototype.text = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.element()];
                    case 1: return [4 /*yield*/, (_a.sent()).text()];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Get web driver element
     */
    UIElement.prototype.element = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this;
                        return [4 /*yield*/, this.refetch()];
                    case 1:
                        _a._element = _b.sent();
                        return [4 /*yield*/, this._element];
                    case 2: return [2 /*return*/, _b.sent()];
                }
            });
        });
    };
    /**
     * Shows if element is displayed. Returns true or false. If the element doesn't exist it will return false
     */
    UIElement.prototype.isDisplayed = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var el, _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.element()];
                    case 1:
                        el = (_b.sent());
                        return [4 /*yield*/, el];
                    case 2:
                        if (!((_b.sent()) === null)) return [3 /*break*/, 3];
                        _a = false;
                        return [3 /*break*/, 5];
                    case 3: return [4 /*yield*/, this._element.isDisplayed()];
                    case 4:
                        _a = (_b.sent());
                        _b.label = 5;
                    case 5: return [2 /*return*/, _a];
                }
            });
        });
    };
    /**
     * Returns true or false
     */
    UIElement.prototype.exists = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, this.element() === null ? false : true];
            });
        });
    };
    /**
     * Waits until the element exists not.
     * @param wait
     */
    UIElement.prototype.waitForExistNot = function (wait) {
        if (wait === void 0) { wait = 3000; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._webio.waitForExist(this._searchParams, wait, true)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Wait until the elements appear
     * @param wait
     */
    UIElement.prototype.waitForExist = function (wait) {
        if (wait === void 0) { wait = 3000; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, this._webio.waitForExist(this._searchParams, wait, false)];
            });
        });
    };
    UIElement.prototype.getAttribute = function (attr) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.element()];
                    case 1: return [4 /*yield*/, (_a.sent()).getAttribute(attr)];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Scroll with offset from elemnt with minimum inertia
     * @param direction
     * @param yOffset
     * @param xOffset
     */
    UIElement.prototype.scroll = function (direction, yOffset, xOffset) {
        if (yOffset === void 0) { yOffset = 0; }
        if (xOffset === void 0) { xOffset = 0; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var location, size, x, y, endPoint, action;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.location()];
                    case 1:
                        location = _a.sent();
                        return [4 /*yield*/, this.size()];
                    case 2:
                        size = _a.sent();
                        x = location.x === 0 ? 10 : location.x;
                        y = (location.y + 15);
                        if (yOffset === 0) {
                            yOffset = location.y + size.y - 15;
                        }
                        if (direction === 0 /* down */) {
                            y = (location.y + size.y) - 15;
                            if (!this._webio.isIOS) {
                                if (yOffset === 0) {
                                    yOffset = location.y + size.y - 15;
                                }
                            }
                        }
                        if (direction === 1 /* up */) {
                            if (yOffset === 0) {
                                yOffset = size.y - 15;
                            }
                        }
                        endPoint = utils_1.calculateOffset(direction, y, yOffset, x, xOffset, this._webio.isIOS, false);
                        if (direction === 0 /* down */) {
                            endPoint.point.y += location.y;
                        }
                        action = new this._wd.TouchAction(this._driver);
                        action
                            .press({ x: x, y: y })
                            .wait(endPoint.duration)
                            .moveTo({ x: endPoint.point.x, y: endPoint.point.y })
                            .release();
                        return [4 /*yield*/, action.perform()];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, this._driver.sleep(150)];
                    case 4:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Scroll with offset from elemnt with minimum inertia
     * @param direction
     * @param yOffset
     * @param xOffset
     */
    UIElement.prototype.scrollTo = function (direction, elementToSearch, yOffset, xOffset) {
        if (yOffset === void 0) { yOffset = 0; }
        if (xOffset === void 0) { xOffset = 0; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var el, retries, _a, error_1;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        el = null;
                        retries = 7;
                        _b.label = 1;
                    case 1:
                        if (!(el === null && retries >= 0)) return [3 /*break*/, 11];
                        _b.label = 2;
                    case 2:
                        _b.trys.push([2, 8, , 10]);
                        return [4 /*yield*/, elementToSearch()];
                    case 3:
                        el = _b.sent();
                        _a = !el || el === null;
                        if (_a) return [3 /*break*/, 5];
                        return [4 /*yield*/, el.isDisplayed()];
                    case 4:
                        _a = !(_b.sent());
                        _b.label = 5;
                    case 5:
                        if (!_a) return [3 /*break*/, 7];
                        el = null;
                        return [4 /*yield*/, this.scroll(direction, yOffset, xOffset)];
                    case 6:
                        _b.sent();
                        _b.label = 7;
                    case 7: return [3 /*break*/, 10];
                    case 8:
                        error_1 = _b.sent();
                        return [4 /*yield*/, this.scroll(direction, yOffset, xOffset)];
                    case 9:
                        _b.sent();
                        return [3 /*break*/, 10];
                    case 10:
                        retries--;
                        return [3 /*break*/, 1];
                    case 11: return [2 /*return*/, el];
                }
            });
        });
    };
    /**
 * Scroll with offset from elemnt with minimum inertia
 * @param direction
 * @param yOffset
 * @param xOffset
 */
    UIElement.prototype.drag = function (direction, yOffset, xOffset) {
        if (xOffset === void 0) { xOffset = 0; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var location, x, y, endPoint, action;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.location()];
                    case 1:
                        location = _a.sent();
                        x = location.x === 0 ? 10 : location.x;
                        y = location.y === 0 ? 10 : location.y;
                        endPoint = utils_1.calculateOffset(direction, y, yOffset, x, xOffset, this._webio.isIOS, false);
                        action = new this._wd.TouchAction(this._driver);
                        action
                            .press({ x: x, y: y })
                            .wait(endPoint.duration)
                            .moveTo({ x: endPoint.point.x, y: endPoint.point.y })
                            .release();
                        return [4 /*yield*/, action.perform()];
                    case 2:
                        _a.sent();
                        return [4 /*yield*/, this._driver.sleep(150)];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    UIElement.prototype.log = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a, _b;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        _b = (_a = console).dir;
                        return [4 /*yield*/, this.element()];
                    case 1:
                        _b.apply(_a, [_c.sent()]);
                        return [2 /*return*/];
                }
            });
        });
    };
    UIElement.prototype.refetch = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var error_2;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 5, , 6]);
                        if (!(this._index && this._index !== null)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this._driver[this._searchMethod](this._searchParams, 1000)];
                    case 1: return [2 /*return*/, (_a.sent())[this._index]];
                    case 2: return [4 /*yield*/, this._driver[this._searchMethod](this._searchParams, 1000)];
                    case 3: return [2 /*return*/, _a.sent()];
                    case 4: return [3 /*break*/, 6];
                    case 5:
                        error_2 = _a.sent();
                        return [2 /*return*/, null];
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    return UIElement;
}());
exports.UIElement = UIElement;
